@@ -4,7 +4,7 @@
 setMethod(
     f = "Deriv",
     signature = signature(x = 'dich', Theta = 'matrix'),
-    definition = function(x, Theta, EM = FALSE, estHess = FALSE, offterm = numeric(1L)){
+    definition = function(x, Theta, estHess = FALSE, offterm = numeric(1L)){
         if(nrow(x@fixed.design) > 1L && ncol(x@fixed.design) > 0L)
             Theta <- cbind(x@fixed.design, Theta)
         ret <- .Call('dparsDich', x@par, Theta, estHess, x@dat, offterm)
@@ -16,11 +16,10 @@ setMethod(
 setMethod(
     f = "Deriv",
     signature = signature(x = 'graded', Theta = 'matrix'),
-    definition = function(x, Theta, EM = FALSE, estHess = FALSE, offterm = numeric(1L)){
+    definition = function(x, Theta, estHess = FALSE, offterm = numeric(1L)){
         if(nrow(x@fixed.design) > 1L && ncol(x@fixed.design) > 0L)
             Theta <- cbind(x@fixed.design, Theta)
-        P <- P.poly(x@par, Theta, ot=offterm)
-        ret <- .Call("dparsPoly", P, Theta, x@dat,
+        ret <- .Call("dparsPoly", x@par, Theta, offterm, x@dat,
             length(x@par) - ncol(Theta), estHess)
         if(x@any.prior) ret <- DerivativePriors(x=x, grad=ret$grad, hess=ret$hess)
         return(ret)
@@ -30,7 +29,7 @@ setMethod(
 setMethod(
     f = "Deriv",
     signature = signature(x = 'rating', Theta = 'matrix'),
-    definition = function(x, Theta, EM = FALSE,  estHess = FALSE, offterm = numeric(1L)){
+    definition = function(x, Theta, estHess = FALSE, offterm = numeric(1L)){
         hess <- matrix(0, length(x@par), length(x@par))
         dat <- x@dat
         nfact <- x@nfact
@@ -42,8 +41,9 @@ setMethod(
         nd <- length(d)
         if(nrow(x@fixed.design) > 1L && ncol(x@fixed.design) > 0L)
             Theta <- cbind(x@fixed.design, Theta)
-        P <- P.poly(c(a, d + shift), Theta, ot=offterm)
-        ret <- .Call("dparsPoly", P, Theta, x@dat,
+        par <- c(a, d + shift)
+        P <- P.poly(par=par, Theta=Theta, ot=offterm)
+        ret <- .Call("dparsPoly", par, Theta, offterm, x@dat,
                      length(d), estHess)
         grad <- ret$grad
         hess <- ret$hess
@@ -93,7 +93,7 @@ setMethod(
 setMethod(
     f = "Deriv",
     signature = signature(x = 'partcomp', Theta = 'matrix'),
-    definition = function(x, Theta, EM = FALSE, estHess = FALSE, offterm = numeric(1L)){
+    definition = function(x, Theta, estHess = FALSE, offterm = numeric(1L)){
         #local derivative from previous version with small mod
         #u and g in logit form
         dpars.comp <- function(lambda,zeta,g,r,f,Thetas,D,estHess)
@@ -105,8 +105,8 @@ setMethod(
                 d <- pars[1L:nfact]
                 a <- pars[(nfact+1L):(length(pars)-1L)]
                 c <- pars[length(pars)]
-                P <- P.comp(c(a,d,c,999), thetas)
-                Pstar <- P.comp(c(a,d,-999,999),thetas)
+                P <- P.comp(c(a,d,c,999), thetas)[,2L]
+                Pstar <- P.comp(c(a,d,-999,999),thetas)[,2L]
                 Qstar <- 1 - Pstar
                 Q <- 1 - P
                 c <- antilogit(c)
@@ -115,7 +115,7 @@ setMethod(
                 dd <- da <- rep(0,nfact)
                 dc <- sum(r/P * (g_1g * (1 - Pstar)) + (f-r)/Q * (g_1g * (Pstar - 1)))
                 for(i in 1L:nfact){
-                    Pk <- P.mirt(c(a[i],d[i],-999,999),matrix(thetas[,i]))
+                    Pk <- P.mirt(c(a[i],d[i],-999,999),matrix(thetas[,i]))[,2L]
                     Qk <- 1 - Pk
                     dd[i] <- sum((1-c)*Pstar*Qk*const1)
                     da[i] <- sum((1-c)*Pstar*Qk*thetas[,i]*const1)
@@ -127,8 +127,8 @@ setMethod(
                 d <- pars[1L:nfact]
                 a <- pars[(nfact+1L):(length(pars)-1L)]
                 c <- pars[length(pars)]
-                P <- P.comp(c(a,d,c,999), thetas)
-                Pstar <- P.comp(c(a,d,-999,999),thetas)
+                P <- P.comp(c(a,d,c,999), thetas)[,2L]
+                Pstar <- P.comp(c(a,d,-999,999),thetas)[,2L]
                 Qstar <- 1 - Pstar
                 Q <- 1 - P
                 g <- c <- antilogit(c)
@@ -146,9 +146,9 @@ setMethod(
                             d2 <- strsplit(Names[c(i,j)],"_")[[2L]]
                             k <- as.numeric(d1[2L])
                             m <- as.numeric(d2[2L])
-                            Pk <- P.mirt(c(a[k],d[k],-999,999),matrix(thetas[,k]))
+                            Pk <- P.mirt(c(a[k],d[k],-999,999),matrix(thetas[,k]))[,2L]
                             Qk <- 1 - Pk
-                            Pm <- P.mirt(c(a[m],d[m],-999,999),matrix(thetas[,m]))
+                            Pm <- P.mirt(c(a[m],d[m],-999,999),matrix(thetas[,m]))[,2L]
                             Qm <- 1 - Pm
                             if(i == j && d1[1L] == 'd'){
                                 hess[i,i] <- sum(r/P * ((1-g)*(Pstar - 3*Pstar*Pk + 2*Pstar*Pk^2) ) -
@@ -254,22 +254,10 @@ setMethod(
 setMethod(
     f = "Deriv",
     signature = signature(x = 'gpcm', Theta = 'matrix'),
-    definition = function(x, Theta, EM = FALSE, estHess = FALSE, offterm = numeric(1L)){
-        dat <- x@dat
-        nfact <- x@nfact
-        nzetas <- ncol(dat)
-        a <- ExtractLambdas(x)
-        d <- ExtractZetas(x)
-        ak <- 0:(length(d)-1L)
+    definition = function(x, Theta, estHess = FALSE, offterm = numeric(1L)){
         if(nrow(x@fixed.design) > 1L && ncol(x@fixed.design) > 0L)
             Theta <- cbind(x@fixed.design, Theta)
-        P <- ProbTrace(x=x, Theta=Theta, useDesign = FALSE, ot=offterm)
-        num <- P.nominal(a=a, ak=ak, d=d, Theta=Theta, returnNum=TRUE, ot=offterm)
-        tmp <- nominalParDeriv(a=a, ak=ak, d=d, Theta=Theta, estHess=estHess,
-                               P=P, num=num, dat=dat, gpcm=TRUE)
-        keep <- rep(TRUE, length(tmp$grad))
-        keep[(nfact+1L):(nfact+length(d))] <- FALSE
-        ret <- list(grad=tmp$grad[keep], hess=tmp$hess[keep, keep])
+        ret <- .Call("dparsNominal", x, Theta, offterm, FALSE, estHess)
         if(x@any.prior) ret <- DerivativePriors(x=x, grad=ret$grad, hess=ret$hess)
         return(ret)
     }
@@ -278,17 +266,13 @@ setMethod(
 setMethod(
     f = "Deriv",
     signature = signature(x = 'nestlogit', Theta = 'matrix'),
-    definition = function(x, Theta, EM = FALSE, estHess = FALSE, offterm = numeric(1L)){
+    definition = function(x, Theta, estHess = FALSE, offterm = numeric(1L)){
         #u and g in logit
         grad <- rep(0, length(x@par))
         hess <- matrix(0, length(x@par), length(x@par))
         dat <- x@dat
-        if(EM){
-            if(estHess)
-                hess[x@est, x@est] <- numDeriv::hessian(EML, x@par[x@est], obj=x, Theta=Theta)
-        } else {
-            hess[x@est, x@est] <- numDeriv::hessian(L, x@par[x@est], obj=x, Theta=Theta)
-        }
+        if(estHess)
+            hess[x@est, x@est] <- numDeriv::hessian(EML, x@par[x@est], obj=x, Theta=Theta)
         nfact <- x@nfact
         a <- x@par[1L:x@nfact]
         d <- x@par[x@nfact+1L]
@@ -297,15 +281,13 @@ setMethod(
         ak <- x@par[(x@nfact+4L):(x@nfact+4L+x@ncat-2L)]
         dk <- x@par[(length(x@par)-length(ak)+1):length(x@par)]
         correct <- x@correctcat
-        if(nrow(x@fixed.design) > 1L && ncol(x@fixed.design) > 0L)
-            Theta <- cbind(x@fixed.design, Theta)
-        Pd <- P.mirt(c(a, d, g, u), Theta=Theta)
+        Pd <- P.mirt(c(a, d, g, u), Theta=Theta)[,2L]
         g <- antilogit(g)
         u <- antilogit(u)
         Qd <- 1 - Pd
-        Pstar <- P.mirt(c(a, d, -999, 999), Theta=Theta)
+        Pstar <- P.mirt(c(a, d, -999, 999), Theta=Theta)[,2L]
         Qstar <- 1 - Pstar
-        num <- P.nominal(a=rep(1, nfact), ak=ak, d=dk, Theta=Theta, returnNum=TRUE)
+        num <- P.nominal(c(rep(1, nfact), ak, dk), ncat=length(dk), Theta=Theta, returnNum=TRUE)
         den <- rowSums(num)
         Pn <- num/den
         cdat <- dat[,correct]
@@ -337,7 +319,7 @@ setMethod(
 setMethod(
     f = "Deriv",
     signature = signature(x = 'rsm', Theta = 'matrix'),
-    definition = function(x, Theta, EM = FALSE, estHess = FALSE, offterm = numeric(1L)){
+    definition = function(x, Theta, estHess = FALSE, offterm = numeric(1L)){
         dat <- x@dat
         nfact <- x@nfact
         nzetas <- ncol(dat)
@@ -350,15 +332,11 @@ setMethod(
         if(nrow(x@fixed.design) > 1L && ncol(x@fixed.design) > 0L)
             Theta <- cbind(x@fixed.design, Theta)
         P <- ProbTrace(x=x, Theta=Theta, useDesign = FALSE, ot=offterm)
-        num <- P.nominal(a=a, ak=ak, d=dshift, Theta=Theta, returnNum=TRUE, ot=offterm)
-        tmp <- nominalParDeriv(a=a, ak=ak, d=dshift, Theta=Theta, estHess=estHess,
-                               P=P, num=num, dat=dat)
-        keep <- rep(TRUE, length(tmp$grad))
-        keep[(nfact+1L):(nfact+length(d))] <- FALSE
-        grad <- c(tmp$grad[keep], 0)
-        hess <- tmp$hess[keep, keep]
-        hess <- cbind(hess, rep(0, nrow(hess)))
-        hess <- rbind(hess, rep(0, ncol(hess)))
+        oldpar <- x@par
+        tmp <- .Call("dparsNominal", x, Theta, offterm, TRUE, estHess)        
+        num <- P.nominal(c(a, ak, dshift), ncat=length(ak), Theta=Theta, returnNum=TRUE, ot=offterm)
+        grad <- tmp$grad
+        hess <- tmp$hess
 
         #quick calcs for derivs
         nfact <- length(a)
@@ -380,32 +358,30 @@ setMethod(
         ak0 <- ak
         ak0[1L] <- 0
         cind <- length(grad)
-        tmp <- 0
-        for(i in 1L:nzetas)
-            tmp <- tmp + dat[,i]*numD^2 / numsum^2 - dat[,i]*numD2/numsum
-        hess[cind, cind] <- sum(tmp)
-        for(j in 1L:nzetas){
+        if(estHess){
             tmp <- 0
             for(i in 1L:nzetas)
-                tmp <- tmp + dat[,i]*P[,j]*numD/numsum - dat[,i]*P[,j]
-            hess[cind, nfact+j] <- hess[nfact+j, cind] <- sum(tmp)
-        }
-        for(j in 1L:nfact){
-            tmp <- 0
-            for(i in 1L:nzetas)
-                tmp <- tmp + dat[,i]*numD*numakThetaD[,j]/numsum^2 -
-                    dat[,i]* (num %*% ak0*Theta[,j])/numsum
-            hess[cind, j] <- hess[j, cind] <- sum(tmp)
+                tmp <- tmp + dat[,i]*numD^2 / numsum^2 - dat[,i]*numD2/numsum
+            hess[cind, cind] <- sum(tmp)
+            for(j in 1L:nzetas){
+                tmp <- 0
+                for(i in 1L:nzetas)
+                    tmp <- tmp + dat[,i]*P[,j]*numD/numsum - dat[,i]*P[,j]
+                hess[cind, nfact+j] <- hess[nfact+j, cind] <- sum(tmp)
+            }
+            for(j in 1L:nfact){
+                tmp <- 0
+                for(i in 1L:nzetas)
+                    tmp <- tmp + dat[,i]*numD*numakThetaD[,j]/numsum^2 -
+                        dat[,i]* (num %*% ak0*Theta[,j])/numsum
+                hess[cind, j] <- hess[j, cind] <- sum(tmp)
+            }
         }
         ####
-        #TEMP - can't seem to get the last value of the gradient quite right for some reason....
+        #FIXME - can't seem to get the last value of the gradient quite right for some reason....
         x2 <- x
         x2@est <- c(rep(FALSE, length(x2@est)-1L), TRUE)
-        if(EM){
-            grad[x2@est] <- numDeriv::grad(EML, x@par[x2@est], obj=x2, Theta=Theta)
-        } else {
-            grad[x2@est] <- numDeriv::grad(L, x@par[x2@est], obj=x2, Theta=Theta, ot=offterm)
-        }
+        grad[x2@est] <- numDeriv::grad(EML, x@par[x2@est], obj=x2, Theta=Theta)
         ####
         ret <- list(grad=grad, hess=hess)
         if(x@any.prior) ret <- DerivativePriors(x=x, grad=ret$grad, hess=ret$hess)
@@ -416,19 +392,10 @@ setMethod(
 setMethod(
     f = "Deriv",
     signature = signature(x = 'nominal', Theta = 'matrix'),
-    definition = function(x, Theta, EM = FALSE, estHess = FALSE, offterm = numeric(1L)){
-        dat <- x@dat
-        nfact <- x@nfact
-        nzetas <- ncol(dat)
-        a <- ExtractLambdas(x)
-        ak <- x@par[(nfact+1L):(nzetas + nfact)]
-        d <- ExtractZetas(x)
+    definition = function(x, Theta, estHess = FALSE, offterm = numeric(1L)){
         if(nrow(x@fixed.design) > 1L && ncol(x@fixed.design) > 0L)
             Theta <- cbind(x@fixed.design, Theta)
-        P <- ProbTrace(x=x, Theta=Theta, useDesign = FALSE, ot=offterm)
-        num <- P.nominal(a=a, ak=ak, d=d, Theta=Theta, returnNum=TRUE, ot=offterm)
-        ret <- nominalParDeriv(a=a, ak=ak, d=d, Theta=Theta, estHess=estHess,
-                               P=P, num=num, dat=dat)
+        ret <- .Call("dparsNominal", x, Theta, offterm, FALSE, estHess)
         if(x@any.prior) ret <- DerivativePriors(x=x, grad=ret$grad, hess=ret$hess)
         return(ret)
     }
@@ -460,7 +427,7 @@ nominalParDeriv <- function(a, ak, d, Theta, P, num, dat, estHess, gpcm = FALSE)
 setMethod(
     f = "Deriv",
     signature = signature(x = 'GroupPars', Theta = 'matrix'),
-    definition = function(x, Theta, EM = FALSE, pars = NULL, itemloc = NULL,
+    definition = function(x, Theta, CUSTOM.IND, EM = FALSE, pars = NULL, itemloc = NULL,
                           tabdata = NULL, prior = NULL, estHess=FALSE){
         if(EM){
             grad <- rep(0, length(x@par))
@@ -469,7 +436,7 @@ setMethod(
                 if(any(x@est)){
                     hess[x@est,x@est] <- numDeriv::hessian(EML2, x@par[x@est], Theta=Theta,
                                                            pars=pars, tabdata=tabdata,
-                                                           itemloc=itemloc)
+                                                           itemloc=itemloc, CUSTOM.IND=CUSTOM.IND)
                 }
                 return(list(grad=grad, hess=hess))
             }
@@ -479,14 +446,15 @@ setMethod(
             r <- tabdata[ ,ncol(tabdata)]
             N <- sum(r)
             tabdata <- tabdata[ ,-ncol(tabdata)]
-            itemtrace <- computeItemtrace(pars=pars, Theta=Theta, itemloc=itemloc)
+            itemtrace <- computeItemtrace(pars=pars, Theta=Theta, itemloc=itemloc, 
+                                          CUSTOM.IND=CUSTOM.IND)
             mu <- x@par[1L:nfact]
             siglong <- x@par[-(1L:nfact)]
             sig <- matrix(0,nfact,nfact)
             selcov <- lower.tri(sig, diag=TRUE)
             scores2 <- matrix(0, nrow(tabdata), sum(selcov))
             thetas2 <- numeric(sum(selcov))
-            ret <- .Call('EAPgroup', itemtrace, tabdata, Theta, prior, mu)
+            ret <- .Call('EAPgroup', log(itemtrace), tabdata, Theta, prior, mu)
             tmp <- cbind(ret$scores, ret$scores2) * r
             newpars <- apply(tmp, 2, sum) / N
             if(nfact > 1L){
@@ -584,40 +552,22 @@ setMethod(
 setMethod(
     f = "Deriv",
     signature = signature(x = 'custom', Theta = 'matrix'),
-    definition = function(x, Theta, EM = FALSE, estHess = FALSE, offterm = numeric(1L)){
+    definition = function(x, Theta, estHess = FALSE, offterm = numeric(1L)){
         if(x@useuserdata) Theta <- cbind(Theta, x@userdata)
         grad <- rep(0, length(x@par))
         hess <- matrix(0, length(x@par), length(x@par))
-        if(EM){
-            if(x@usegr) grad <- x@gr(x, Theta)
-            else grad[x@est] <- numDeriv::grad(EML, x@par[x@est], obj=x, Theta=Theta)
-            if(estHess){
-                if(x@usehss) hess <- x@hss(x, Theta)
-                else hess[x@est, x@est] <- numDeriv::hessian(EML, x@par[x@est], obj=x,
-                                                             Theta=Theta)
-            }
-            return(list(grad = grad, hess=hess))
-        }
         if(x@usegr) grad <- x@gr(x, Theta)
-        else grad[x@est] <- numDeriv::grad(L, x@par[x@est], obj=x, Theta=Theta)
-        if(x@usehss) hess <- x@hss(x, Theta)
-        else hess[x@est, x@est] <- numDeriv::hessian(L, x@par[x@est], obj=x, Theta=Theta)
-        return(list(grad=grad, hess=hess))
+        else grad[x@est] <- numDeriv::grad(EML, x@par[x@est], obj=x, Theta=Theta)
+        if(estHess){
+            if(x@usehss) hess <- x@hss(x, Theta)
+            else hess[x@est, x@est] <- numDeriv::hessian(EML, x@par[x@est], obj=x,
+                                                         Theta=Theta)
+        }
+        return(list(grad = grad, hess=hess))
     }
 )
 
 #----------------------------------------------------------------------------
-L <- function(par, obj, Theta, ot=numeric(1)){
-    obj@par[obj@est] <- par
-    P <- ProbTrace(obj, Theta, useDesign = FALSE, ot=ot)
-    LL <- obj@dat * P
-    LL[LL < .Machine$double.eps] <- 1
-    LL <- sum(log(LL))
-    if(obj@any.prior)
-        LL <- LL.Priors(x=obj, LL=LL)
-    return(LL)
-}
-
 EML <- function(par, obj, Theta){
     obj@par[obj@est] <- par
     itemtrace <- ProbTrace(x=obj, Theta=Theta)
@@ -626,7 +576,7 @@ EML <- function(par, obj, Theta){
     return(LL)
 }
 
-EML2 <- function(x, Theta, pars, tabdata, itemloc){
+EML2 <- function(x, Theta, pars, tabdata, itemloc, CUSTOM.IND){
     obj <- pars[[length(pars)]]
     obj@par[obj@est] <- x
     r <- tabdata[, ncol(tabdata)]
@@ -635,7 +585,8 @@ EML2 <- function(x, Theta, pars, tabdata, itemloc){
     sigma <- gpars$gcov
     prior <- mvtnorm::dmvnorm(Theta, mean=mu, sigma=sigma)
     prior <- prior/sum(prior)
-    rlist <- Estep.mirt(pars=pars, tabdata=tabdata, Theta=Theta, prior=prior, itemloc=itemloc)
+    rlist <- Estep.mirt(pars=pars, tabdata=tabdata, Theta=Theta, prior=prior, itemloc=itemloc,
+                        CUSTOM.IND=CUSTOM.IND)
     LL <- sum(r*log(rlist$expected))
     LL <- LL.Priors(x=obj, LL=LL)
     return(LL)
@@ -660,7 +611,7 @@ setMethod(
         g <- antilogit(x@par[parlength - 1L])
         d <- x@par[parlength - 2L]
         a <- x@par[1L:nfact]
-        Pstar <- P.mirt(c(a, d, -999, 999), Theta)
+        Pstar <- P.mirt(c(a, d, -999, 999), Theta)[,2L]
         grad <- hess <- vector('list', 2L)
         grad[[1L]] <- grad[[2L]] <- hess[[1L]] <- hess[[2L]] <- matrix(0, N, nfact)
         for(i in 1L:nfact){
@@ -733,14 +684,14 @@ setMethod(
         g <- x@par[parlength - 1L]
         d <- ExtractZetas(x)
         a <- ExtractLambdas(x)
-        P <- P.comp(c(a, d, g, 999), Theta)
-        Pstar <- P.comp(c(a, d, -999, 999), Theta)
+        P <- P.comp(c(a, d, g, 999), Theta)[,2L]
+        Pstar <- P.comp(c(a, d, -999, 999), Theta)[,2L]
         g <- antilogit(g)
         u <- antilogit(u)
         grad <- hess <- vector('list', 2L)
         grad[[1L]] <- grad[[2L]] <- hess[[1L]] <- hess[[2L]] <- matrix(0, N, nfact)
         for(j in 1L:nfact){
-            Pn <- P.mirt(c(a[j], d[j], -999, 999), Theta)
+            Pn <- P.mirt(c(a[j], d[j], -999, 999), Theta)[,2L]
             grad[[2L]][ ,j] <- (u-g)*Pstar*a[j]*(1 - Pn)
             grad[[1L]][ ,j] <- -1*grad[[2L]][ ,j]
             hess[[2L]][ ,j] <- (u-g)*a[j]^2*Pstar*(1 - 3*Pn + 2*Pn^2)
@@ -757,8 +708,8 @@ setMethod(
         a <- ExtractLambdas(x)
         d <- ExtractZetas(x)
         ak <- 0:(x@ncat - 1L)
-        P <- P.nominal(a=a, ak=ak, d=d, Theta=Theta)
-        Num <- P.nominal(a=a, ak=ak, d=d, Theta=Theta, returnNum = TRUE)
+        P <- P.nominal(c(a, ak, d), ncat=length(d), Theta=Theta)
+        Num <- P.nominal(c(a, ak, d), ncat=length(d), Theta=Theta, returnNum = TRUE)
         Den <- rowSums(Num)
         grad <- hess <- vector('list', x@ncat)
         for(i in 1L:x@ncat)
@@ -786,8 +737,8 @@ setMethod(
         d <- d[-length(d)]
         d[-1L] <- d[-1L] + t
         ak <- 0:(x@ncat - 1L)
-        P <- P.nominal(a=a, ak=ak, d=d, Theta=Theta)
-        Num <- P.nominal(a=a, ak=ak, d=d, Theta=Theta, returnNum = TRUE)
+        P <- P.nominal(c(a, ak, d), ncat=length(d), Theta=Theta)
+        Num <- P.nominal(c(a, ak, d), ncat=length(d), Theta=Theta, returnNum = TRUE)
         Den <- rowSums(Num)
         grad <- hess <- vector('list', x@ncat)
         for(i in 1L:x@ncat)
@@ -812,8 +763,8 @@ setMethod(
         a <- ExtractLambdas(x)
         d <- ExtractZetas(x)
         ak <- x@par[(x@nfact+1):(x@nfact+x@ncat)]
-        P <- P.nominal(a=a, ak=ak, d=d, Theta=Theta)
-        Num <- P.nominal(a=a, ak=ak, d=d, Theta=Theta, returnNum = TRUE)
+        P <- P.nominal(c(a, ak, d), ncat=length(d), Theta=Theta)
+        Num <- P.nominal(c(a, ak, d), ncat=length(d), Theta=Theta, returnNum = TRUE)
         Den <- rowSums(Num)
         grad <- hess <- vector('list', x@ncat)
         for(i in 1L:x@ncat)
@@ -841,11 +792,11 @@ setMethod(
         u <- x@par[x@nfact+3L]
         ak <- x@par[(x@nfact+4L):(x@nfact+4L+x@ncat-2L)]
         dk <- x@par[(length(x@par)-length(ak)+1):length(x@par)]
-        Pn <- P.nominal(a=rep(1,ncol(Theta)), ak=ak, d=dk, Theta=Theta)
-        Num <- P.nominal(a=rep(1,ncol(Theta)), ak=ak, d=dk, Theta=Theta, returnNum = TRUE)
+        Pn <- P.nominal(c(rep(1,ncol(Theta)), ak, dk), ncat=length(dk), Theta=Theta)
+        Num <- P.nominal(c(rep(1,ncol(Theta)), ak, dk), ncat=length(dk), Theta=Theta, returnNum = TRUE)
         Den <- rowSums(Num)
-        Pstar <- P.mirt(c(a, d, -999, 999), Theta)
-        Q <- 1 - P.mirt(c(a, d, g, u), Theta)
+        Pstar <- P.mirt(c(a, d, -999, 999), Theta)[,2L]
+        Q <- P.mirt(c(a, d, g, u), Theta)[,1]
         g <- antilogit(g)
         u <- antilogit(u)
         Num2 <- P <- matrix(0, nrow(Theta), x@ncat)
@@ -885,3 +836,17 @@ setMethod(
     }
 )
 
+###
+
+setMethod(
+    f = "dP",
+    signature = signature(x = 'dich', Theta = 'matrix', prior = 'numeric'),
+    definition = function(x, Theta, prior, extra_term = 1){
+        P <- ProbTrace(x, Theta)
+        PQ <- apply(P, 1, prod) 
+        ret <- c(colSums(Theta * PQ * extra_term * prior), 
+                 sum(PQ * extra_term * prior), 
+                 colSums(P * extra_term * prior))
+        ret
+    }
+)
\ No newline at end of file
