@@ -44,10 +44,11 @@
 #'   for more details
 #' @author Phil Chalmers \email{rphilip.chalmers@@gmail.com}
 #' @seealso \code{\link{anova-method}}, \code{\link{coef-method}}, \code{\link{summary-method}},
-#'   \code{\link{residuals-method}}, \code{\link{plot-method}}, \code{\link{fitted-method}},
+#'   \code{\link{residuals-method}}, \code{\link{plot-method}}, 
 #'   \code{\link{expand.table}}, \code{\link{key2binary}}, \code{\link{mirt.model}}, \code{\link{mirt}},
 #'   \code{\link{bfactor}}, \code{\link{multipleGroup}}, \code{\link{mixedmirt}},
-#'   \code{\link{wald}}, \code{\link{itemplot}}, \code{\link{fscores}}, \code{\link{fitIndices}},
+#'   \code{\link{wald}}, \code{\link{itemplot}}, \code{\link{fscores}}, 
+#'   \code{\link{M2}},
 #'   \code{\link{extract.item}}, \code{\link{iteminfo}}, \code{\link{testinfo}}, \code{\link{probtrace}},
 #'   \code{\link{boot.mirt}}, \code{\link{imputeMissing}}, \code{\link{itemfit}}, \code{\link{mod2values}},
 #'   \code{\link{simdata}}, \code{\link{createItem}}
@@ -69,11 +70,6 @@
 #' \emph{Applied Psychological Measurement, 31}, 4-19
 #'
 #' @keywords models
-#' @usage
-#' bfactor(data, model, model2 = mirt.model(paste0('G = 1-', ncol(data))),
-#' SE = FALSE, SE.type = 'SEM', group = NULL, verbose = TRUE, ...)
-#'
-#'
 #' @export bfactor
 #' @examples
 #'
@@ -155,6 +151,7 @@
 #' # Two-tier model
 #'
 #' #simulate data
+#' set.seed(1234)
 #' a <- matrix(c(
 #' 0,1,0.5,NA,NA,
 #' 0,1,0.5,NA,NA,
@@ -178,7 +175,6 @@
 #'
 #' sigma <- diag(5)
 #' sigma[1,2] <- sigma[2,1] <- .4
-#' set.seed(1234)
 #' dataset <- simdata(a,d,2000,itemtype=items,sigma=sigma)
 #'
 #' specific <- c(rep(1,5),rep(2,6),rep(3,5))
@@ -187,14 +183,14 @@
 #'     G2 = 9-16
 #'     COV = G1*G2')
 #'
-#' simmod <- bfactor(dataset, specific, model, quadpts = 9, technical = list(TOL = 1e-3))
+#' simmod <- bfactor(dataset, specific, model, quadpts = 9, TOL = 1e-3)
 #' coef(simmod)
 #' summary(simmod)
 #'
 #'     }
 #'
 bfactor <- function(data, model, model2 = mirt.model(paste0('G = 1-', ncol(data))),
-                    SE = FALSE, SE.type = 'SEM', group = NULL, verbose = TRUE, ...)
+                    SE = FALSE, SE.type = 'crossprod', group = NULL, verbose = TRUE, ...)
 {
     Call <- match.call()
     if(!is.numeric(model))
@@ -213,7 +209,7 @@ bfactor <- function(data, model, model2 = mirt.model(paste0('G = 1-', ncol(data)
     if(is.null(group)) group <- rep('all', nrow(data))
     mod <- ESTIMATION(data=data, model=model, group=group,
                       method = 'EM', verbose=verbose,
-                      BFACTOR = TRUE, SE=SE, ...)
+                      BFACTOR = TRUE, SE=SE, SE.type=SE.type, ...)
     if(is(mod, 'ConfirmatoryClass') || is(mod, 'MultipleGroupClass'))
         mod@Call <- Call
     return(mod)
