@@ -7,19 +7,24 @@
 #'
 #' @aliases itemfit
 #' @param x a computed model object of class \code{ExploratoryClass}, \code{ConfirmatoryClass}, or
-#' \code{MultipleGroupClass}
+#'   \code{MultipleGroupClass}
 #' @param Zh logical; calculate Zh and associated statistics (infit/outfit)? Disable this is you are only
-#' interested in computing the S-X2 quickly
+#'   interested in computing the S-X2 quickly
 #' @param X2 logical; calculate the X2 statistic for unidimensional models?
 #' @param mincell the minimum expected cell size to be used in the S-X2 computations. Tables will be
-#' collapsed across items first if polytomous, and then across scores if necessary
+#'   collapsed across items first if polytomous, and then across scores if necessary
 #' @param S_X2.tables logical; return the tables in a list format used to compute the S-X2 stats?
 #' @param group.size approximate size of each group to be used in calculating the \eqn{\chi^2} statistic
 #' @param empirical.plot a single numeric value or character of the item name  indicating which item to plot
-#'  (via \code{itemplot}) and
-#' overlay with the empirical \eqn{\theta} groupings. Only applicable when \code{type = 'X2'}.
-#' The default is \code{NULL}, therefore no plots are drawn
+#'   (via \code{itemplot}) and overlay with the empirical \eqn{\theta} groupings. Only applicable
+#'   when \code{type = 'X2'}. The default is \code{NULL}, therefore no plots are drawn
+#' @param empirical.CI a numeric value indicating the width of the empirical confidence interval ranging 
+#'   between 0 and 1 (default of 0 plots not interval). For example, a 95% confidence interval would be 
+#'   plotted if \code{empirical.CI = .95}. Only applicable to dichotomous items
 #' @param method type of factor score estimation method. See \code{\link{fscores}} for more detail
+#' @param Theta a matrix of factor scores used for statistics that require emperical estimates. If 
+#'   supplied, arguments typically passed to \code{fscores()} will be ignored and these values will
+#'   be used instead
 #' @param ... additional arguments to be passed to \code{fscores()}
 #' @author Phil Chalmers \email{rphilip.chalmers@@gmail.com}
 #' @keywords item fit
@@ -62,8 +67,12 @@
 #' fit
 #'
 #' itemfit(x, empirical.plot = 1) #empirical item plot
+#' itemfit(x, empirical.plot = 1, empirical.CI = .99) #empirical item plot with 99% CI's
 #' #method='ML' agrees better with eRm package
 #' itemfit(raschfit, method = 'ML') #infit and outfit stats
+#' #same as above, but inputting ML estimates instead
+#' Theta <- fscores(raschfit, method = 'ML', full.scores=TRUE, scores.only=TRUE)
+#' itemfit(raschfit, Theta=Theta)
 #'
 #' #similar example to Kang and Chen 2007
 #' a <- matrix(c(.8,.4,.7, .8, .4, .7, 1, 1, 1, 1))
@@ -88,7 +97,7 @@
 #'   }
 #'
 itemfit <- function(x, Zh = TRUE, X2 = FALSE, group.size = 150, mincell = 1, S_X2.tables = FALSE,
-                    empirical.plot = NULL, method = 'EAP', ...){
+                    empirical.plot = NULL, empirical.CI = 0, method = 'EAP', Theta = NULL, ...){
     if(any(is.na(x@data)))
         stop('Fit statistics cannot be computed when there are missing data.')
     if(is(x, 'MultipleGroupClass')){
@@ -96,7 +105,7 @@ itemfit <- function(x, Zh = TRUE, X2 = FALSE, group.size = 150, mincell = 1, S_X
         for(g in 1L:length(x@cmods)){
             x@cmods[[g]]@itemtype <- x@itemtype
             ret[[g]] <- itemfit(x@cmods[[g]], group.size=group.size, mincell = 1,
-                                S_X2.tables = FALSE, method=method, ...)
+                                S_X2.tables = FALSE, method=method, Theta=Theta, ...)
         }
         names(ret) <- x@groupNames
         return(ret)
@@ -107,11 +116,12 @@ itemfit <- function(x, Zh = TRUE, X2 = FALSE, group.size = 150, mincell = 1, S_X
     itemloc <- x@itemloc
     pars <- x@pars
     if(Zh || X2){
-        sc <- fscores(x, verbose = FALSE, full.scores = TRUE, method=method, ...)
+        if(is.null(Theta))
+            Theta <- fscores(x, verbose=FALSE, full.scores=TRUE, 
+                             scores.only=TRUE, method=method, ...)
         prodlist <- attr(pars, 'prodlist')
         nfact <- x@nfact + length(prodlist)
         fulldata <- x@fulldata
-        Theta <- sc[ ,ncol(sc):(ncol(sc) - nfact + 1L), drop = FALSE]
         if(method %in% c('ML', 'WLE')){
             for(i in 1L:ncol(Theta)){
                 tmp <- Theta[,i]
@@ -146,7 +156,7 @@ itemfit <- function(x, Zh = TRUE, X2 = FALSE, group.size = 150, mincell = 1, S_X
                 oneslopes[i] <- closeEnough(x@pars[[i]]@par[1L], 1-1e-10, 1+1e-10)
             if(all(oneslopes)){
                 attr(x, 'inoutfitreturn') <- TRUE
-                pf <- personfit(x, method=method, sc=sc)
+                pf <- personfit(x, method=method, Theta=Theta)
                 z2 <- pf$resid^2 / pf$W
                 outfit <- colSums(z2) / N
                 q.outfit <- sqrt(colSums((pf$C / pf$W^2) / N^2) - 1 / N)
@@ -186,11 +196,8 @@ itemfit <- function(x, Zh = TRUE, X2 = FALSE, group.size = 150, mincell = 1, S_X
                 ind <- 1L:length(inames)
                 empirical.plot <- ind[inames == empirical.plot]
             }
-            P <- ProbTrace(pars[[empirical.plot]], ThetaFull)
-            plot(ThetaFull, P[,1], type = 'l', ylim = c(0,1), las = 1,
-                 main =paste('Item', empirical.plot), ylab = expression(P(theta)), xlab = expression(theta))
-            for(i in 2L:ncol(P))
-                lines(ThetaFull, P[,i], col = i)
+            empirical.plot_P <- ProbTrace(pars[[empirical.plot]], ThetaFull)
+            empirical.plot_points <- matrix(NA, length(unique(Groups)), x@K[empirical.plot] + 2L)
         }
         for (i in 1L:J){
             if(!is.null(empirical.plot) && i != empirical.plot) next
@@ -201,8 +208,7 @@ itemfit <- function(x, Zh = TRUE, X2 = FALSE, group.size = 150, mincell = 1, S_X
                 mtheta <- matrix(mean(Theta[Groups == j,]), nrow=1)
                 if(!is.null(empirical.plot)){
                     tmp <- r/N
-                    col <- 2:length(tmp)
-                    points(rep(mtheta, length(tmp) - 1), tmp[-1], col = col, pch = col+2)
+                    empirical.plot_points[j, ] <- c(mtheta, N, tmp)
                 }
                 P <- ProbTrace(x=pars[[i]], Theta=mtheta)
                 if(any(N * P < 2)){
@@ -214,7 +220,44 @@ itemfit <- function(x, Zh = TRUE, X2 = FALSE, group.size = 150, mincell = 1, S_X
             df[i] <- df[i] + n.uniqueGroups*(length(r) - 1) - sum(pars[[i]]@est)
         }
         X2[X2 == 0] <- NA
-        if(!is.null(empirical.plot)) return(invisible(NULL))
+        if(!is.null(empirical.plot)){
+            K <- x@K[empirical.plot]
+            EPCI.lower <- EPCI.upper <- NULL
+            if(K == 2 && empirical.CI != 0){
+                p.L <- function(x, alpha) if (x[1] == 0) 0 else qbeta(alpha, x[1], x[2] - x[1] + 1)
+                p.U <- function(x, alpha) if (x[1] == x[2]) 1 else qbeta(1 - alpha, x[1] + 1, x[2] - x[1])
+                N <- empirical.plot_points[,2]
+                O <- empirical.plot_points[,ncol(empirical.plot_points)] * N
+                EPCI.lower <- apply(cbind(O, N), 1, p.L, (1-empirical.CI)/2)
+                EPCI.upper <- apply(cbind(O, N), 1, p.U, (1-empirical.CI)/2)
+            }
+            empirical.plot_points <- empirical.plot_points[,-2]
+            colnames(empirical.plot_points) <- c('theta', paste0('p.', 1:K))
+            while(nrow(empirical.plot_points) < nrow(empirical.plot_P))
+                empirical.plot_points <- rbind(empirical.plot_points, 
+                                               rep(NA, length(empirical.plot_points[1,])))
+            plt.1 <- data.frame(id = 1:nrow(ThetaFull), Theta=ThetaFull, P=empirical.plot_P)
+            plt.1 <- reshape(plt.1, varying = 3:ncol(plt.1), direction = 'long', timevar = 'cat')
+            plt.2 <- data.frame(id = 1:nrow(empirical.plot_points), empirical.plot_points)
+            plt.2 <- reshape(plt.2, varying = 3:ncol(plt.2), direction = 'long', timevar = 'cat')
+            plt <- cbind(plt.1, plt.2)
+            if(K == 2) plt <- plt[plt$cat != 1, ]
+            return(xyplot(P ~ Theta, plt, group = cat, 
+                          main = paste('Empirical plot for item', empirical.plot), ylim = c(-0.1,1.1),
+                          xlab = expression(theta), ylab=expression(P(theta)), 
+                          auto.key=ifelse(K==2, FALSE, TRUE), EPCI.lower=EPCI.lower,
+                          EPCI.upper=EPCI.upper,
+                          panel = function(x, y, groups, subscripts, EPCI.lower, EPCI.upper, ...){
+                              panel.xyplot(x=x, y=y, groups=groups, type='l', subscripts=subscripts, ...)
+                              panel.points(cbind(plt$theta, plt$p), col=groups, pch=groups, ...)     
+                              if(!is.null(EPCI.lower)){
+                                  theta <- na.omit(plt$theta)
+                                  for(i in 1:length(theta))
+                                      panel.lines(c(theta[i], theta[i]), c(EPCI.lower[i], EPCI.upper[i]),
+                                                  lty = 2, col = 'red')
+                              }
+                          }))
+        }
         ret$X2 <- X2
         ret$df <- df
         ret$p.X2 <- round(1 - pchisq(X2, df), 4)
@@ -324,7 +367,7 @@ itemfit <- function(x, Zh = TRUE, X2 = FALSE, group.size = 150, mincell = 1, S_X
         S_X2 <- df.S_X2 <- numeric(J)
         O <- makeObstables(dat, x@K)
         Nk <- rowSums(O[[1L]])
-        E <- EAPsum(x, S_X2 = TRUE, gp = list(gmeans=0, gcov=matrix(1)))
+        E <- EAPsum(x, S_X2 = TRUE, gp = list(gmeans=0, gcov=matrix(1)), CUSTOM.IND=x@CUSTOM.IND)
         for(i in 1L:J)
             E[[i]] <- E[[i]] * Nk
         coll <- collapseCells(O, E, mincell=mincell)
