@@ -73,6 +73,8 @@ mod2values <- function(x){
     }
     gnames <- rep(names(PrepList), each = length(est)/length(PrepList))
     par[parname %in% c('g', 'u')] <- antilogit(par[parname %in% c('g', 'u')])
+    prior.type <- sapply(as.character(prior.type), 
+                         function(x) switch(x, '1'='norm', '2'='lnorm', '3'='beta', 'none'))
     ret <- data.frame(group=gnames, item=item, class=class, name=parname, parnum=parnum, value=par,
                       lbound=lbound, ubound=ubound, est=est, prior.type=prior.type,
                       prior_1=prior_1, prior_2=prior_2)
